;**********************************************************************
;    Filename:	selec.asm 
;    Date:          	2005/1/1
;    File Version:
;**********************************************************************

	list      		p=16F88			 ; list directive to define processor
	#include		 <p16F88.inc>		 ; processor specific variable definitions

	__CONFIG   _CONFIG1,  _CP_OFF & _WDT_OFF & _PWRTE_ON & _INTRC_IO & _LVP_OFF 

;***** VARIABLE DEFINITIONS
w_temp			EQU     0x20			 ; variable used for context saving 
status_temp		EQU     0x21			 ; variable used for context saving

wait_data		EQU	0x30		;wait data ram
taihi			EQU	0x31		;taihi data
sel_data			EQU	0x32		;SELECTER EEPROM 10H

wait_data_2		EQU	0x39		;wait data 2
key_data			EQU	0x3a		;key data ram
									;key_data,0  SW1
									;key_data,1  SW2
									;key_data,2  SW3
									;key_data,3  SW4
key_input		EQU	0x3b		;key input ram
									;key_input,0  SW1
									;key_input,1  SW2
									;key_input,2  SW3
									;key_input,3  SW4

ee_data			EQU	0x3f		;EEPROM data

tm0_setdata		EQU	0x63		;FFH-63H=156 156*128=19.968msec

;**********************************************************************
		ORG     		0x000             ; processor reset vector
  		goto    		main              ; go to beginning of program

;----------------------------------------------------------------------------------
		ORG     		0x004             ; interrupt vector location
		movwf   	w_temp            ; save off current W register contents
		movf		STATUS,w          ; move status register into W register
		movwf		status_temp       ; save off contents of STATUS register

		;---------------------------------	
		bcf			INTCON,TMR0IF
		movlw		tm0_setdata
		movwf		TMR0


next_int
		movf    		status_temp,w     ; retrieve copy of STATUS register
		movwf		STATUS            ; restore pre-isr STATUS register contents
		swapf   		w_temp,f
		swapf  		 w_temp,w          ; restore pre-isr W register contents
		retfie                    ; return from interrupt
;---------------------------------------------------------------------------------------
main
		call			osc_set
		call			port_set
		call			ad_set
		
		movlw		20h
		movwf		PORTA
		movlw		0ffh
		movwf		PORTB

		call			timer0_set

		call			shoki_set
		call			sel_set
;--------------------------------------
main_loop
		movlw		0ffh				;key input clear
		movwf		key_input
		btfss		PORTB,4		;SW1 check
		bcf			key_input,0		;SW1 input ari
		btfss		PORTB,5		;SW2 check
		bcf			key_input,1		;SW2 input ari		
		btfss		PORTB,6		;SW3 check
		bcf			key_input,2		;SW3 input ari
		btfss		PORTB,7		;SW4 check
		bcf			key_input,3		
		;--------------------------------- 
		call		wait_10
		;----------------------------------SW1 check
		btfss		PORTB,4		;SW1 key check
		goto		sw1_next
		bsf			key_data,0		;SW1 off
		goto		sw1_end
sw1_next
		btfsc		key_input,0		;SW1 input check
		goto		sw1_end
		btfss		key_data,0		;SW1 on check
		goto		sw1_end
		bcf			key_data,0		;SW1 on
		;------------------------
		movlw		00h
		movwf		sel_data
		bsf			sel_data,0
		nop
		call			sel_set
		call			eeprom_set
		goto		sw_end

sw1_end
		;----------------------------------SW2 check
		btfss		PORTB,5		;SW2 key check
		goto		sw2_next
		bsf			key_data,1		;SW2 off
		goto		sw2_end
sw2_next
		btfsc		key_input,1		;SW2 input check
		goto		sw2_end
		btfss		key_data,1		;SW2 on check
		goto		sw2_end
		bcf			key_data,1		;SW2 on
		;------------------------
		movlw		00h
		movwf		sel_data
		bsf			sel_data,1
		nop
		call			sel_set
		call			eeprom_set
		goto		sw_end

sw2_end
		;----------------------------------SW3 check
		btfss		PORTB,6		;SW3 key check
		goto		sw3_next
		bsf			key_data,2		;SW3 off
		goto		sw3_end
sw3_next
		btfsc		key_input,2		;SW3 input check
		goto		sw3_end
		btfss		key_data,2		;SW3 on check
		goto		sw3_end
		bcf			key_data,2		;SW3 on
		;------------------------
		movlw		00h
		movwf		sel_data
		bsf			sel_data,2
		nop
		call			sel_set
		call			eeprom_set
		goto		sw_end

sw3_end
		;----------------------------------SW4 check
		btfss		PORTB,7		;SW4 key check
		goto		sw4_next
		bsf			key_data,3		;SW4 off
		goto		sw4_end
sw4_next
		btfsc		key_input,3		;SW4 input check
		goto		sw4_end
		btfss		key_data,3		;SW4 on check
		goto		sw4_end
		bcf			key_data,3		;SW4 on
		;------------------------
		movlw		00h
		movwf		sel_data
		bsf			sel_data,3
		nop
		call			sel_set
		call			eeprom_set
		goto		sw_end

sw4_end

sw_end
	
;---------------------------------------
		goto		main_loop
;-------------------------------------------------------------------------------
wait_20
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		return
;-------------------------------------------------------------------------------
wait_10
		movlw		5h
		movwf		wait_data_2
wait_10_2
		call			wait_1
		decfsz		wait_data_2,1
		goto		wait_10_2
		return

wait_0
		movlw		0ffh
		movwf		wait_data
		goto		wait_2
wait_1		
		movlw		0ffh
		movwf		wait_data
wait_2
		nop
		decfsz		wait_data,1
		goto		wait_2
		return
;------------------------------------------------------------------------------
led_all_off
		bsf			PORTB,0
		nop
		bsf			PORTB,1
		nop
		bsf			PORTB,2
		nop
		bsf			PORTB,3
		nop
		return
;------------------------------------------------------------------------------
shoki_set
		movlw		00h
		call			eeprom_read
		sublw		96h				;HANTEI DATA
		btfss		STATUS,2
		goto		shoki_set2


		movlw		10h
		call			eeprom_read
		movwf		sel_data
		return

shoki_set2
		call			led_all_off
		bcf			PORTB,0
		nop
		bsf			sel_data,0
		nop
		call			sel_set
		call			eeprom_set
		return
;-------------------------------------------------------------------------------
sel_set
		call			led_all_off
		btfsc		sel_data,0
		goto		sel_set0
		btfsc		sel_data,1
		goto		sel_set1
		btfsc		sel_data,2
		goto		sel_set2
		btfsc		sel_data,3
		goto		sel_set3
		return
sel_set0
		bcf			PORTA,0
		nop
		bcf			PORTA,1
		nop
		bcf			PORTB,0
		nop
		return
sel_set1
		bsf			PORTA,0
		nop
		bcf			PORTA,1
		nop
		bcf			PORTB,1
		nop
		return
sel_set2
		bcf			PORTA,0
		nop
		bsf			PORTA,1
		nop
		bcf			PORTB,2
		nop
		return
sel_set3
		bsf			PORTA,0
		nop
		bsf			PORTA,1
		nop
		bcf			PORTB,3
		nop
		return
;-------------------------------------------------------------------------------
eeprom_set
		movf		sel_data,0
		movwf		ee_data
		movlw		10h
		call			eeprom_write
		
eeprom_id
		movlw		96h
		movwf		ee_data
		movlw		00h
		call			eeprom_write
		return

eeprom_read
		call			banksel_2
		movwf		EEADR
		call			banksel_3
		bsf			EECON1,RD
		call			banksel_2
		movf		EEDATA,0
		call			banksel_0		
		return

eeprom_write
		call			banksel_2
		movwf		EEADR
		call			banksel_0
		movf		ee_data,0
		call			banksel_2
		movwf		EEDATA
		call			banksel_3
		bsf			EECON1,WREN
		movlw		55h
		movwf		EECON2
		movlw		0aah
		movwf		EECON2
		bsf			EECON1,WR
eeprom_write2
		btfsc		EECON1,WR
		goto		eeprom_write2
		call			banksel_0
		return
;------------------------------------------------------------------------------
banksel_0
		bcf			STATUS,RP0
		nop
		bcf			STATUS,RP1
		nop
		return
banksel_1
		bsf			STATUS,RP0
		nop
		bcf			STATUS,RP1
		nop
		return
banksel_2
		bcf			STATUS,RP0
		nop
		bsf			STATUS,RP1
		nop
		return
banksel_3
		bsf			STATUS,RP0
		nop
		bsf			STATUS,RP1
		nop
		return
;------------------------------------------------------------------------------
osc_set
		call			banksel_1
								;(0)NC
								;(110)IRCF2-IRCF0
								;(0)OSTS
								;(1)IOFS
								;(00)SCS
		movlw		64h
		movwf		OSCCON
		call			banksel_0
		return
;-------------------------------------------------------------------------------
port_set							;RA1,RA2,RA3,RA4,RA6,RA7(OUT)
								;RB0,RB1,RB2,RB3(OUT)
								;RB4,RB5,RB6,RB7(IN)
		call			banksel_1
		clrf			TRISA
		nop
		clrf			TRISB
		bsf			TRISB,4
		nop
		bsf			TRISB,5
		nop
		bsf			TRISB,6
		nop
		bsf			TRISB,7
		nop
		call			banksel_0
		return
;------------------------------------------------------------------------------
ad_set
								;(01)ADCS1,ADCS0
								;(000)CHS2,CHS1,CHS0
								;(0)GO/DONE
								;(0)NC
								;(0)ADON
		movlw		40h
		movwf		ADCON0
		call			banksel_1
								;(0)ADFM
								;(1)ADCS2
								;(00)VCFG1,VCFG0
								;(0000)
		movlw		040h
		movwf		ADCON1
								;(0)NC
								;(0000000)ANS6-ANS0
		movlw		00h
		movwf		ANSEL
		call			banksel_0
		return
;------------------------------------------------------------------------------
timer0_set

		call			banksel_1
		movlw		086h		;(1)PORTB NOT PULLUP
								;(0)INTEDG
								;(0)TOCS
								;(0)TOSE
								;(0)PSA
								;(110)1/128
		movwf		OPTION_REG
		call			banksel_0

		movlw		tm0_setdata
		movwf		TMR0
		bcf			INTCON,TMR0IE
		bcf			INTCON,GIE
		return
;-------------------------------------------------------------------------------

		end                     ; directive 'end of program'
